//-----------------------------------------------------------------------------
/*!
\file TcpDataReceiver.cpp
\brief Source for DataReceiver.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpDataReceiver.h"
#include "EuException.h"
#include <sstream>
#include <cstring>
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
#ifdef _DEBUG
  #include <iostream>
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER Receiver
  //---------------------------------------------------------------------------
  DataReceiver::DataReceiver()
  : SUPER(),
    m_uiDataSize( DEFAULT_DATASIZE ),
    m_uiStreamSize(0),
    m_uiStreamPosistion(0),
    m_szStreamBuffer( nullptr ),
    m_pszSentBuffer( nullptr )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "DataReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  DataReceiver::DataReceiver( SOCKET Socket )
  : SUPER( Socket ),
    m_uiDataSize( DEFAULT_DATASIZE ),
    m_uiStreamSize(0),
    m_uiStreamPosistion(0),
    m_szStreamBuffer( nullptr ),
    m_pszSentBuffer( nullptr )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "DataReceiver";
    #endif
  }

  //----------------------------------------------------------------------
  DataReceiver::DataReceiver( TcpMain * pParent, SOCKET Socket )
  : SUPER( pParent, Socket ),
    m_uiDataSize( DEFAULT_DATASIZE ),
    m_uiStreamSize(0),
    m_uiStreamPosistion(0),
    m_szStreamBuffer( nullptr ),
    m_pszSentBuffer( nullptr )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "DataReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  DataReceiver::DataReceiver( TcpMain * pParent, SOCKET Socket, unsigned int uiDataSize )
  : SUPER( pParent, Socket ),
    m_uiDataSize( uiDataSize ),
    m_uiStreamSize(0),
    m_uiStreamPosistion(0),
    m_szStreamBuffer( nullptr ),
    m_pszSentBuffer( nullptr )
  {
    if ( ! uiDataSize )
    {
      throw eut::ErrorNException( "DataReceiver ctor()", ERR_NO_DATASIZE, __FILE__, __LINE__ );
    }
    #ifdef _DEBUG
      m_strDebugInfo = "DataReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  DataReceiver::~DataReceiver()
  {
    if ( m_szStreamBuffer ) delete[] m_szStreamBuffer;
    if ( m_pszSentBuffer ) delete[] m_pszSentBuffer;
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class Receiver' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  DataReceiver::DataReceiver( const DataReceiver & )
  : m_uiDataSize(DEFAULT_DATASIZE),
    m_uiStreamSize(0),
    m_uiStreamPosistion(0),
    m_szStreamBuffer( nullptr ),
    m_pszSentBuffer( nullptr )
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class Receiver' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  void DataReceiver::setDataSize( unsigned int uiDataSize )
  {
    m_uiDataSize = uiDataSize;
  }
  //---------------------------------------------------------------------------
  unsigned int DataReceiver::getDataSize() const
  {
    return m_uiDataSize;
  }
  //---------------------------------------------------------------------------
  void DataReceiver::initStreamData()
  {
    if ( m_szStreamBuffer )
    {
        delete[] m_szStreamBuffer;
        m_szStreamBuffer = nullptr;
    }
    m_uiStreamSize = 0;
    m_uiStreamPosistion = 0;
  }
  //---------------------------------------------------------------------------
  int DataReceiver::processReceivedData( char * pszSentBuffer, unsigned int t_uiSentBufferSize )
  {
    int                t_iErr = 0;
    AbstractMessage * t_pTcpMessage( nullptr );

    // wenn noch kein Stream, auswerten ob zuständig und ob zu erwartende Größe übergeben
    if ( m_szStreamBuffer == nullptr )
    {
      // Größe des Buffers muss mindesten die Länge von t_szKey und einer
      // einstelligen Zahl plus terminierender Nullen sein...
      // ... und ist das ein Msg, welche uns betrifft?
      char  t_szKey[] = "hja-tcp";
      if ( t_uiSentBufferSize < 10 || strcmp( t_szKey, pszSentBuffer ) != 0 )
      {
        this->generateNewTcpMessage(  t_pTcpMessage,
                                      nullptr, // RawMsg
                                      0,  // RawMsgSize
                                      0,  // MsgID
                                      ERR_WRONG_STREAMTYPE, // ActionCode
                                      "not hja-tcp" );  // Info
        // wird im Parent wieder gelöscht!
        t_iErr = this->acceptMessage( t_pTcpMessage );
      }
      else
      {
        // zu erwartende Streamgröße ermitteln...
        std::istringstream ( pszSentBuffer + 8 ) >> m_uiStreamSize;
        // ...und Streambuffer dementsprechend initialisieren
        m_szStreamBuffer = new char[m_uiStreamSize];
      }
    }
    // wenn Streamdaten übernommen werden können
    if ( m_uiStreamSize )
    {
      // gesendete Daten übernehmen, wenn nicht zu groß
      if ( t_uiSentBufferSize > ( m_uiStreamSize - m_uiStreamPosistion ) )
      {
        unsigned int    t_uiRestBufferSize = m_uiStreamSize - m_uiStreamPosistion,
                        t_uiToMuchBufferSize = t_uiSentBufferSize - t_uiRestBufferSize;

        // Den zu vielen Speicher retten, da der gesendete Buffer gelöscht wird
        char  * t_szToMuchBuffer = new char[t_uiToMuchBufferSize];
        std::memcpy( t_szToMuchBuffer, pszSentBuffer + t_uiRestBufferSize, t_uiToMuchBufferSize );
        // den vorderen Teil in t_pTcpMessage und versenden
        std::memcpy( m_szStreamBuffer + m_uiStreamPosistion, pszSentBuffer, t_uiRestBufferSize );
        this->generateNewTcpMessage(  t_pTcpMessage,
                                      m_szStreamBuffer, // SendMsg
                                      m_uiStreamSize ); // SendMsgSize
        m_szStreamBuffer = nullptr;
        this->initStreamData();
        t_iErr = this->acceptMessage( t_pTcpMessage );
        if ( ! t_iErr )
        {
          t_iErr = this->processReceivedData( t_szToMuchBuffer, t_uiToMuchBufferSize );
        }
        delete[] t_szToMuchBuffer;
      }
      else
      {
        std::memcpy( m_szStreamBuffer + m_uiStreamPosistion, pszSentBuffer, t_uiSentBufferSize );
        m_uiStreamPosistion += t_uiSentBufferSize;
        // wenn alle Daten vorhanden..
        if ( m_uiStreamSize == m_uiStreamPosistion )
        {
          this->generateNewTcpMessage(  t_pTcpMessage,
                                        m_szStreamBuffer, // SendMsg
                                        m_uiStreamSize ); // SendMsgSize
          // Daten sind jetzt in t_pTcpMessage und werden dort entfernt
          m_szStreamBuffer = nullptr;
          this->initStreamData();
          // wird im Parent wieder gelöscht!
          t_iErr = this->acceptMessage( t_pTcpMessage );
        }
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  DataReceiver & DataReceiver::operator= ( const DataReceiver & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
