//-----------------------------------------------------------------------------
/*!
\file TcpReceiverTransmitter.cpp
\brief Source for ReceiverTransmitter.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpReceiverTransmitter.h"
#include "EuException.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
#ifdef _DEBUG
  #include <iostream>
  #include <sstream>
#endif
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  /*
   * WinSock 2 extension -- manifest constants for shutdown()
   */
  #define SD_RECEIVE      0x00
  #define SD_SEND         0x01
  #define SD_BOTH         0x02
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  ReceiverTransmitter::ReceiverTransmitter()
  : m_Socket(0)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::ReceiverTransmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  ReceiverTransmitter::ReceiverTransmitter( SOCKET Socket )
  : m_Socket( Socket )
  {
    if ( ! Socket )
    {
      throw eut::ErrorNException( "tcp:ReceiverTransmitter ctor()", ERR_PARAMETER_NULL_FORBIDDEN, __FILE__, __LINE__ );
    }
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::ReceiverTransmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  ReceiverTransmitter::~ReceiverTransmitter()
  {
    if ( m_Socket )
    {
      #if defined(_MSC_VER) || defined(__MINGW32__)
        shutdown( m_Socket, SD_BOTH );
        closesocket( m_Socket );
      #elif defined(__GNUC__)
        shutdown( m_Socket, SHUT_RDWR );
        close( m_Socket );
      #endif
      m_Socket = 0;
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class EuEndlessPThread' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  ReceiverTransmitter::ReceiverTransmitter( const ReceiverTransmitter & )
  : m_Socket(0)
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
  // Warnung: base class 'class EuEndlessPThread' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
    int ReceiverTransmitter::takeABreak ()
  #else
    void ReceiverTransmitter::takeABreak ()
  #endif
  {
    throw eut::ErrorNException( "ReceiverTransmitter::takeABreak()", ERR_METHOD_FORBIDDEN, __FILE__, __LINE__ );
  }
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
    int ReceiverTransmitter::finishBreak ()
  #else
    void ReceiverTransmitter::finishBreak ()
  #endif
  {
    throw eut::ErrorNException( "ReceiverTransmitter::finishBreak()", ERR_METHOD_FORBIDDEN, __FILE__, __LINE__ );
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // warning: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wold-style-cast"
    // warning: cast from A to B increases required alignment from 1 to 4 [-Wcast-align]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wcast-align"
  #endif
  //---------------------------------------------------------------------------
  int ReceiverTransmitter::getPeerName( std::string & rstrIPAdress, unsigned int & ruiPort ) const
  {
    int           t_iErr = 0;
    SOCKADDR_IN   t_Peer;
    #if defined(_MSC_VER) || defined(__MINGW32__)
      int         t_iPeer_len;
    #elif defined(__GNUC__)
      socklen_t   t_iPeer_len;
    #endif

    t_iPeer_len = sizeof( t_Peer );
    if ( getpeername( m_Socket, (struct sockaddr*) & t_Peer, &t_iPeer_len) == -1)
    {
      t_iErr = ERR_GETPEERNAME_FAILED;
    }
    else
    {
      rstrIPAdress = inet_ntoa( t_Peer.sin_addr );
      ruiPort = static_cast<unsigned int> ( ntohs( t_Peer.sin_port ) );
      if ( rstrIPAdress == "127.0.0.1" || rstrIPAdress == "LOCALHOST" )
      {
        char    t_HostName[255];
        
        if ( gethostname( t_HostName, sizeof( t_HostName ) ) == 0 )
        {
          #if defined(_MSC_VER) || defined(__MINGW32__)
            PHOSTENT          t_HostInfo;
          #elif defined(__GNUC__)
            struct hostent  * t_HostInfo;
          #endif

          if ( ( t_HostInfo = gethostbyname( t_HostName ) ) != nullptr )
          {
            rstrIPAdress = inet_ntoa( * ( struct in_addr * ) * t_HostInfo->h_addr_list );
          }
        }
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // warning: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic pop
    // warning: cast from A to B increases required alignment from 1 to 4 [-Wcast-align]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  SOCKET ReceiverTransmitter::Socket () const
  {
    return m_Socket;
  }
  //---------------------------------------------------------------------------
  ReceiverTransmitter & ReceiverTransmitter::operator= ( const ReceiverTransmitter & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
