//-----------------------------------------------------------------------------
/*!
\file TcpTransmitter.cpp
\brief Source for Transmitter.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTransmitter.h"
#include "EuException.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNABORTED 10053L
  #define WSAECONNRESET 10054L
  #define WSAESHUTDOWN 10058L
#endif
#include <sstream>
#ifdef _DEBUG
  #include <iostream>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER ReceiverTransmitter
  //---------------------------------------------------------------------------
  Transmitter::Transmitter ()
  : SUPER()
  {
    #ifdef _DEBUG
      m_strDebugInfo = "Transmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  Transmitter::Transmitter (SOCKET Socket)
  : SUPER( Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "Transmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  Transmitter::~Transmitter()
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class ReceiverTransmitter' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  Transmitter::Transmitter( const Transmitter & )
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class ReceiverTransmitter' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsequence-point"
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-conversion"
  #endif
  //---------------------------------------------------------------------------
  void Transmitter::work()
  {
    Message  * t_pMessage = nullptr;

    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
    #endif
    this->waitForInput();
    #if defined (__PTHREADS)
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    //^^^^^^^^^^^^^^^^^^^^
    #if defined (__PTHREADS)
      if ( m_eThreadOrder != EuPTHREAD_TAKE_A_BREAK )
    #else
      if ( this->getStatus() == eWORKING )
    #endif
    {
      //++++++++++++++++++++++++++++
      this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( ! m_MsgQueue.empty() )
      {
        t_pMessage = m_MsgQueue.front();
        m_MsgQueue.pop();
      }
      //++++++++++++++++++++++++++++
      this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( t_pMessage )
      {
        int   t_iSendBytes = 0; // Anzahl Bytes der gesendeten t_pMessage
        bool  t_bNOTSOCK = false;

        do
        {
          t_iSendBytes += send( m_Socket,
                                t_pMessage->getTransmitDataPt() + t_iSendBytes,
                                static_cast<int> ( t_pMessage->getTransmitDataSize() ) - t_iSendBytes, 0 );
          if( t_iSendBytes < 0 ) // Wenn send einen Wert < 0 zurück gibt, deutet dies auf einen Fehler hin.
          {
            #if defined(_MSC_VER) || defined(__MINGW32__)
              int t_iErr = WSAGetLastError();
            #elif defined(__GNUC__)
              int t_iErr = errno;
            #endif
            if ( t_iErr == WSAECONNABORTED || t_iErr == WSAESHUTDOWN || t_iErr == WSAENOTSOCK || t_iErr == WSAECONNRESET || t_iErr == EBADF)
            {
              t_bNOTSOCK = true;
            }
            else
            {
              // wenn positiver Wert aus WSAGetLastError() hier auf Minus
              t_iErr = ( t_iErr > 0 ? t_iErr = t_iErr * -1 : t_iErr );
              std::ostringstream      t_ossThrowOut;
              t_ossThrowOut << "Transmitter::work() -> send Error: " << t_iErr;
              throw eut::ErrorNException(t_ossThrowOut.str(), t_iErr, __FILE__, __LINE__ );
            }
          }
        } while( ! t_bNOTSOCK && t_iSendBytes < static_cast<int> ( t_pMessage->getTransmitDataSize() ) );
        delete t_pMessage;
      }
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic pop
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
  void Transmitter::cleanUp ()
  {
    SUPER::cleanUp();
    while ( ! m_MsgQueue.empty() )
    {
      delete m_MsgQueue.front();
      m_MsgQueue.pop();
    }
  }
  //---------------------------------------------------------------------------
  #else
  //---------------------------------------------------------------------------
  void Transmitter::cancel()
  {
    SUPER::cancel();
    while ( ! m_MsgQueue.empty() )
    {
      delete m_MsgQueue.front();
      m_MsgQueue.pop();
    }
  }
  #endif
  //---------------------------------------------------------------------------
  int Transmitter::acceptOrder (Message * pTcpMessage)
  {
    int t_iErr = 0;

    #if defined (__PTHREADS)
      if ( this->status() != EuPTHREAD_WORKING )
      {
        t_iErr = EuPTHREAD_ERR_NO_THREAD;
      }
    #else
      if ( this->getStatus() != eWORKING )
      {
        t_iErr = EuTHREAD_ERR_NO_THREAD;
      }
    #endif
    else
    {
      //++++++++++++++++++++++++++++
        this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
        m_MsgQueue.push( pTcpMessage );
        this->signalInput();
      //++++++++++++++++++++++++++++
        this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
      #ifdef _DEBUG_THREAD
        std::cout << "Transmitter::acceptOrder(...) - Auftrag uebernommen" << std::endl;
      #endif
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  Transmitter & Transmitter::operator= ( const Transmitter & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
